#include "command_overrides.hpp"

#include <unordered_map>
#include <unordered_set>
#include "hook.hpp"

static std::unordered_set<CommandKey> hidden_commands;
static std::unordered_map<CommandKey, POINT> command_positions;
static bool default_hotkeys_disabled = true;

static TCCommandBar_AddCommand orig_CCommandBar_AddCommand;

CommandKey::CommandKey(const CCommandButtonData& data) {
	ability_id = data.ability_id;
	int i = get_command_index(data, LEFT_MOUSE_BUTTON);
	order_id = data.order_id[i != -1 ? i : 0];
}

bool CommandKey::operator == (const CommandKey& other) const {
	return (ability_id == other.ability_id) && (order_id == other.order_id);
}

bool CommandKey::operator != (const CommandKey& other) const {
	return !(*this == other);
}

void set_command_hidden(const CommandKey& key, bool state) {
	if (state) {
		hidden_commands.insert(key);
	}
	else {
		hidden_commands.erase(key);
	}
}

bool is_command_hidden(const CommandKey& key) {
	return hidden_commands.find(key) != hidden_commands.end();
}

void reset_hidden_commands() {
	hidden_commands.clear();
}

void set_command_position(const CommandKey& key, const POINT& position) {
	command_positions[key] = position;
}

bool reset_command_position(const CommandKey& key) {
	return command_positions.erase(key);
}

bool get_command_position(const CommandKey& key, POINT& position) {
	auto it = command_positions.find(key);
	if (it != command_positions.end()) {
		position = it->second;
		return true;
	}

	return false;
}

void set_default_hotkeys_disabled(bool state) {
	default_hotkeys_disabled = state;
}

void __thiscall hook_CCommandBar_AddCommand(CCommandBar* self, CCommandButtonData* data) {
	if (!is_command_hidden(*data)) {
		POINT position;
		if (get_command_position(*data, position)) {
			data->x = position.x;
			data->y = position.y;
		}

		if (default_hotkeys_disabled) {
			data->hotkey = -1;
		}

		orig_CCommandBar_AddCommand(self, data);
	}
}

bool command_overrides_initialize() {
	return hook(
		"CCommandBar::AddCommand",
		(void*)CCommandBar_AddCommand,
		(void*)hook_CCommandBar_AddCommand,
		(void**)&orig_CCommandBar_AddCommand
	);
}

bool command_overrides_finalize() {
	return unhook(
		"CCommandBar::AddCommand",
		(void*)CCommandBar_AddCommand
	);
}
