#pragma once

#include <functional>
#include <optional>
#include <windef.h>

namespace hotkey_system {
	enum KeyModifier {
		ALT = 1,
		CTRL = 2,
		SHIFT = 4,
		WIN = 8
	};

	const int ALL_MODIFIERS = ALT | CTRL | SHIFT | WIN;

	struct Hotkey {
		DWORD key;
		int modifiers;

		size_t operator == (const Hotkey& other) const;
		size_t operator != (const Hotkey& other) const;
	};

	typedef bool (* HotkeyCallback)(Hotkey& hotkey, int extra_modifiers, void* parameter);

	bool emit_key_event(DWORD key, DWORD flags);
	bool connect(Hotkey& hotkey, HotkeyCallback callback, int modifiers_mask = ALL_MODIFIERS, void* parameter = nullptr);
	bool disconnect(Hotkey& hotkey, HotkeyCallback callback, std::optional<void*> parameter = {});
	bool attach_to_window(HWND window);
	bool detach_from_current_window();
	bool initialize();
	bool finalize();
}

template<> struct std::hash<hotkey_system::Hotkey> {
	size_t operator () (const hotkey_system::Hotkey& hotkey) const {
		std::hash<DWORD> hasher;
		return hasher(hotkey.key) ^ hasher(hotkey.modifiers);
	}
};
