#pragma once

#include "nlohmann/json.hpp"

using json = nlohmann::json;

bool json_parse_file(const char* file_path, json& root);
const char* json_type_name(json::value_t type);
bool json_check_type(const json& data, json::value_t type, const char* fmt, ...);
bool json_check_index(const json& array, int index, const char* fmt, ...);
bool json_get_field(const char* array_name, const json& array, int index, json& field);
bool json_get_field(const char* array_name, const json& array, int index, json& field, json::value_t type);
bool json_get_field(const json& object, const char* field_name, json& field);
bool json_get_field(const json& object, const char* field_name, json& field, json::value_t type);

template<typename T>
struct to_json_type_id {};

#define MAP_TO_JSON_TYPE_ID(TYPE, JSON_TYPE) \
	template<> \
	struct to_json_type_id<TYPE> { \
		static const json::value_t value = JSON_TYPE; \
	};

// MAP_TO_JSON_TYPE_ID(?, json::value_t::null)
// MAP_TO_JSON_TYPE_ID(?, json::value_t::object)
// MAP_TO_JSON_TYPE_ID(?, json::value_t::array)
MAP_TO_JSON_TYPE_ID(std::string, json::value_t::string)
MAP_TO_JSON_TYPE_ID(const char*, json::value_t::string)
MAP_TO_JSON_TYPE_ID(bool, json::value_t::boolean)
MAP_TO_JSON_TYPE_ID(signed int, json::value_t::number_integer)
MAP_TO_JSON_TYPE_ID(unsigned int, json::value_t::number_unsigned)
MAP_TO_JSON_TYPE_ID(float, json::value_t::number_float)
MAP_TO_JSON_TYPE_ID(double, json::value_t::number_float)
MAP_TO_JSON_TYPE_ID(long double, json::value_t::number_float)
// MAP_TO_JSON_TYPE_ID(?, json::value_t::binary)
// MAP_TO_JSON_TYPE_ID(?, json::value_t::discarded)

template<typename T>
bool json_get_field(const json& object, const char* field_name, T& value) {
	json field;
	if (!json_get_field(object, field_name, field)
		|| !json_check_type(field, to_json_type_id<T>::value, field_name)
	) {
		return false;
	}

	value = field.get<T>();
	return true;
}
