#include <errhandlingapi.h>
#include <processthreadsapi.h>
#include <synchapi.h>
#include <sysinfoapi.h>
#include <windef.h>
#include <winnt.h>
#include "hook.hpp"
#include "wc3/game_base.hpp"
#include "hotkey_parser.hpp"
#include "hotkey_system.hpp"
#include "command_overrides.hpp"
#include "command_hotkeys.hpp"
#include "wc3/wc3props.hpp"
#include "wc3/input_events.hpp"

static void set_tick_handler_enabled(bool enabled);

static BOOL __fastcall on_tick(void* event_data, void* parameter) {
	hotkey_system::attach_to_window(*p_game_window);
	set_tick_handler_enabled(false);
	return TRUE;
}

static bool tick_handler_registered = false;

static void set_tick_handler_enabled(bool enabled) {
	if (enabled) {
		if (!tick_handler_registered) {
			register_event_handler(EVT_TYPE::TICK, on_tick, nullptr, 0);
		}
	}
	else {
		if (tick_handler_registered) {
			unregister_event_handler(EVT_TYPE::TICK, on_tick);
		}
	}

	tick_handler_registered = enabled;
}

static bool init() {
	if (!TlsGetValue(*gvar_wc3_props_tls)) {
		TlsSetValue(*gvar_wc3_props_tls, (void*)*gvar_wc3_props);
	}
	set_tick_handler_enabled(true);
	return true;
}

static bool finalize() {
	set_tick_handler_enabled(false);
	return true;
}

BOOL APIENTRY DllMain(HMODULE module, UINT reason, LPVOID reserved) {
	switch (reason) {
		case DLL_PROCESS_ATTACH:
			setvbuf(stdout, nullptr, _IONBF, 0);
			setvbuf(stderr, nullptr, _IONBF, 0);

			return hook_initialize()
				&& hotkey_parser_initialize()
				&& hotkey_system::initialize()
				&& init()
				&& command_overrides_initialize()
				&& command_hotkeys_initialize()
				;
			break;
		case DLL_PROCESS_DETACH:
			return command_overrides_finalize()
				&& command_hotkeys_finalize()
				&& finalize()
				&& hotkey_system::finalize()
				&& hook_finalize()
				;
			break;
		default:
			return TRUE;
	}
}
